# coding=utf-8
# 此脚本为os_baseline系统基线检测和修复的脚本
import os, re,sys
import platform
import json
import pwd


class check(object):

    def __init__(self):
        self.update_func = ''
        self.checkdata = {}
        self.updata_result = {}
        self.func_result = []

    def check_max_user_loggin(self):
        result = {}
        result["value"] = "update_max_user_loggin"
        result["msg"] = "确保SSH MaxAuthTries设置为4或更低(小)"
        num = 0
        try:
            with open("/etc/ssh/sshd_config") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line.startswith("MaxAuthTries 4"):
                        num += 1
            if num > 0:
                result["code"] = 0
            else:
                result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    def password_lock(self):
        result = {}
        result["value"] = "update_password_lock"
        result["msg"] = "确保配置了密码尝试失败的锁定"
        password_auth_num = 0
        try:
            with open("/etc/pam.d/password-auth") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line == "auth required pam_faillock.so preauth audit silent deny=5 unlock_time=900" or line == "auth [success=1 default=bad] pam_unix.so" or line == "auth [default=die] pam_faillock.so authfail audit deny=5 unlock_time=900" or line == "auth sufficient pam_faillock.so authsucc audit deny=5 unlock_time=900":
                        password_auth_num += 1
            with open("/etc/pam.d/system-auth") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line == "auth required pam_faillock.so preauth audit silent deny=5 unlock_time=900" or line == "auth [success=1 default=bad] pam_unix.so" or line == "auth [default=die] pam_faillock.so authfail audit deny=5 unlock_time=900" or line == "auth sufficient pam_faillock.so authsucc audit deny=5 unlock_time=900":
                        password_auth_num += 1
            if password_auth_num == 8:
                result["code"] = 0
            else:
                result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    def shell_TMOUT(self):
        result = {}
        result["value"] = "update_shell_TMOUT"
        result["msg"] = "确保默认用户shell超时为900秒或更短"
        num = 0
        try:
            with open("/etc/profile") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line == "TMOUT=600":
                        num += 1
            with open("/etc/bashrc") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line == "TMOUT=600":
                        num += 1
            if num == 2:
                result["code"] = 0
            else:
                result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    def no_passwd(self):
        result = {}
        result["value"] = "update_no_passwd"
        result["msg"] = "确保已禁用SSH空密码登录"
        num = 0
        try:
            with open("/etc/ssh/sshd_config") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line == "PermitEmptyPasswords no":
                        num += 1
            if num > 0:
                result["code"] = 0
            else:
                result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    def ssh_timeout(self):
        result = {}
        result["value"] = "update_ssh_timeout"
        result["msg"] = "确保已配置SSH空闲超时间隔"
        num = 0
        try:
            with open("/etc/ssh/sshd_config") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line == "ClientAliveInterval 300" or line == "ClientAliveCountMax 0":
                        num += 1
            if num >= 2:
                result["code"] = 0
            else:
                result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    def mem_check(self):
        result = {}
        result["value"] = "update_mem_check"
        result["msg"] = "确保核心转储受到限制"
        num = 0
        try:
            with open("/etc/security/limits.conf") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line == "* hard core 0":
                        num += 1
            with open("/etc/sysctl.conf") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line == "fs.suid_dumpable = 0":
                        num += 1
            if num == 2:
                result["code"] = 0
            else:
                result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    def grub_file_user_sys(self):
        result = {}
        result["value"] = "update_grub_file_user_sys"
        result["msg"] = "确保配置了bootloader配置的权限"
        sys_version = self.get_sys()
        try:
            if sys_version.startswith("7"):
                user = pwd.getpwuid(os.stat('/boot/grub2/grub.cfg').st_uid)[0]
                limits = os.popen("stat -c %a /boot/grub2/grub.cfg").read()
                if user == "root" and int(limits.strip('\n')) <= 600:
                    result["code"] = 0
                else:
                    result["code"] = 1
            else:
                user = pwd.getpwuid(os.stat('/boot/grub/grub.conf').st_uid)[0]
                limits = os.popen("stat -c %a /boot/grub/grub.conf").read()
                if user == "root" and int(limits.strip('\n')) <= 600:
                    result["code"] = 0
                else:
                    result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    def open_ASLR(self):
        result = {}
        result["value"] = "update_open_ASLR"
        result["msg"] = "确保启用了地址空间布局随机化（ASLR）"
        num = 0
        try:
            with open("/etc/sysctl.conf") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line == "kernel.randomize_va_space = 2":
                        num += 1
            if num == 1:
                result["code"] = 0
            else:
                result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    def session_loggings_sys(self):
        result = {}
        result["value"] = "update_session_loggings_sys"
        result["msg"] = "确保收集了会话启动信息"
        sys_version = self.get_sys()
        num = 0
        try:
            if sys_version.startswith("7"):
                with open("/etc/audit/rules.d/audit.rules") as f:
                    for line in f.readlines():
                        line = line.strip()
                        if line == "-w /var/run/utmp -p wa -k session" or line == "-w /var/log/wtmp -p wa -k logins" or line == "-w /var/log/btmp -p wa -k logins":
                            num += 1
                if num == 3:
                    result["code"] = 0
                else:
                    result["code"] = 1
            else:
                with open("/etc/audit/audit.rules") as f:
                    for line in f.readlines():
                        line = line.strip()
                        if line == "-w /var/run/utmp -p wa -k session" or line == "-w /var/log/wtmp -p wa -k session" or line == "-w /var/log/btmp -p wa -k session":
                            num += 1
                if num == 3:
                    result["code"] = 0
                else:
                    result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    def admin_log_sys(self):
        result = {}
        result["value"] = "update_admin_log_sys"
        result["msg"] = "确保收集了系统管理员操作（sudolog）"
        sys_version = self.get_sys()
        num = 0
        try:
            if sys_version.startswith("7"):
                with open("/etc/audit/rules.d/audit.rules") as f:
                    for line in f.readlines():
                        line = line.strip()
                        if line == "-w /var/log/sudo.log -p wa -k actions":
                            num += 1
                if num == 1:
                    result["code"] = 0
                else:
                    result["code"] = 1
            else:
                with open("/etc/audit/audit.rules") as f:
                    for line in f.readlines():
                        line = line.strip()
                        if line == "-w /var/log/sudo.log -p wa -k actions":
                            num += 1
                if num == 1:
                    result["code"] = 0
                else:
                    result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    def logging_exit_log_sys(self):
        result = {}
        result["value"] = "update_logging_exit_log_sys"
        result["msg"] = "确保收集了登录和注销事件"
        sys_version = self.get_sys()
        num = 0
        try:
            if sys_version.startswith("7"):
                with open("/etc/audit/rules.d/audit.rules") as f:
                    for line in f.readlines():
                        line = line.strip()
                        if line == "-w /var/log/lastlog -p wa -k logins" or line == "-w /var/run/faillock/ -p wa -k logins":
                            num += 1
                if num == 2:
                    result["code"] = 0
                else:
                    result["code"] = 1
            else:
                with open("/etc/audit/audit.rules") as f:
                    for line in f.readlines():
                        line = line.strip()
                        if line == "-w /var/log/lastlog -p wa -k logins" or line == "-w /var/run/faillock/ -p wa -k logins":
                            num += 1
                if num == 2:
                    result["code"] = 0
                else:
                    result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    def scope_log_sys(self):
        result = {}
        result["value"] = "update_scope_log_sys"
        result["msg"] = "确保系统管理范围（sudoers）更改的收集"
        sys_version = self.get_sys()
        num = 0
        try:
            if sys_version.startswith("7"):
                with open("/etc/audit/rules.d/audit.rules") as f:
                    for line in f.readlines():
                        line = line.strip()
                        if line == "-w /etc/sudoers -p wa -k scope" or line == "-w /etc/sudoers.d/ -p wa -k scope":
                            num += 1
                if num == 2:
                    result["code"] = 0
                else:
                    result["code"] = 1
            else:
                with open("/etc/audit/audit.rules") as f:
                    for line in f.readlines():
                        line = line.strip()
                        if line == "-w /etc/sudoers -p wa -k scope" or line == "-w /etc/sudoers.d -p wa -k scope":
                            num += 1
                if num == 2:
                    result["code"] = 0
                else:
                    result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    def max_log_no_del(self):
        result = {}
        result["value"] = "update_max_log_no_del"
        result["msg"] = "确保审核日志不会自动删除"
        num = 0
        try:
            with open("/etc/audit/auditd.conf") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line == "max_log_file_action = keep_logs":
                        num += 1
            if num == 1:
                result["code"] = 0
            else:
                result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    def user_group_log_sys(self):
        result = {}
        result["value"] = "update_user_group_log_sys"
        result["msg"] = "确保收集了修改用户/组信息的事件"
        sys_version = self.get_sys()
        num = 0
        try:
            if sys_version.startswith("7"):
                with open("/etc/audit/rules.d/audit.rules") as f:
                    for line in f.readlines():
                        line = line.strip()
                        if line == "-w /etc/group -p wa -k identity" or line == "-w /etc/passwd -p wa -k identity" or line == "-w /etc/gshadow -p wa -k identity" or line == "-w /etc/shadow -p wa -k identity" or line == "-w /etc/security/opasswd -p wa -k identity":
                            num += 1
                if num == 5:
                    result["code"] = 0
                else:
                    result["code"] = 1
            else:
                with open("/etc/audit/audit.rules") as f:
                    for line in f.readlines():
                        line = line.strip()
                        if line == "-w /etc/group -p wa -k identity" or line == "-w /etc/passwd -p wa -k identity" or line == "-w /etc/gshadow -p wa -k identity" or line == "-w /etc/shadow -p wa -k identity" or line == "-w /etc/security/opasswd -p wa -k identity":
                            num += 1
                if num == 5:
                    result["code"] = 0
                else:
                    result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    def selinux_wirte_sys(self):
        result = {}
        result["value"] = "update_selinux_wirte_sys"
        result["msg"] = "确保收集了修改系统的强制访问控制的事件"
        sys_version = self.get_sys()
        num = 0
        try:
            if sys_version.startswith("7"):
                with open("/etc/audit/rules.d/audit.rules") as f:
                    for line in f.readlines():
                        line = line.strip()
                        if line == "-w /etc/selinux/ -p wa -k MAC-policy" or line == "-w /usr/share/selinux/ -p wa -k MAC-policy":
                            num += 1
                if num == 2:
                    result["code"] = 0
                else:
                    result["code"] = num
            else:
                with open("/etc/audit/audit.rules") as f:
                    for line in f.readlines():
                        line = line.strip()
                        if line == "-w /etc/selinux/ -p wa -k MAC-policy":
                            num += 1
                if num == 1:
                    result["code"] = 0
                else:
                    result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    def passwd_file_sys6(self):
        result = {}
        result["value"] = "update_passwd_file_sys6"
        result["msg"] = "确保已配置/etc/passwd-的权限"
        try:
            user = pwd.getpwuid(os.stat('/etc/passwd-').st_uid)[0]
            limits = os.popen("stat -c %a /etc/passwd-").read()
            if user == "root" and int(limits.strip('\n')) <= 600:
                result["code"] = 0
            else:
                result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    def group_file_sys6(self):
        result = {}
        result["value"] = "update_group_file_sys6"
        result["msg"] = "确保已配置/etc/group-的权限"
        try:
            user = pwd.getpwuid(os.stat('/etc/group-').st_uid)[0]
            limits = os.popen("stat -c %a /etc/group-").read()
            if user == "root" and int(limits.strip('\n')) <= 600:
                result["code"] = 0
            else:
                result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    def audit_conf_no_auditctl_sys(self):
        result = {}
        result["value"] = "update_audit_conf_no_auditctl_sys"
        result["msg"] = "确保审核配置是不变的"
        sys_version = self.get_sys()
        try:
            if sys_version.startswith("7"):
                with open("/etc/audit/rules.d/audit.rules") as f:
                    lines = f.readlines()
                    if lines[-1].strip('\n') == "-e 2":
                        result["code"] = 0
                    else:
                        result["code"] = 1
            else:
                with open("/etc/audit/audit.rules") as f:
                    lines = f.readlines()
                    if lines[-1].strip('\n') == "-e 2":
                        result["code"] = 0
                    else:
                        result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    def close_cron_mail(self):
        result = {}
        result["value"] = "update_close_cron_mail"
        result["msg"] = "确保定时任务不发送邮件"
        num = 0
        try:
            if os.path.exists("/etc/crontab"):
                with open("/etc/crontab") as f:
                    for line in f.readlines():
                        line = line.strip()
                        if line == 'MAILTO=""':
                            num += 1
                if num == 1:
                    result["code"] = 0
                else:
                    result["code"] = 1
            else:
                result["code"] = 1
        except:
            result["code"] = 1
        self.func_result.append(result)

    def check_umask(self):
        result = {}
        result["value"] = "update_check_umask"
        result["msg"] = "确保默认用户umask限制为027或更高"
        num = 0
        try:
            with open("/etc/profile") as f:
                for line in f.readlines():
                    line = line.strip()
                    if re.search(r"umask \d{3}", line):
                        if int(re.search(r"umask \d{3}", line).group().split(" ")[1]) >= 27:
                            num += 1
            with open("/etc/bashrc") as f:
                for line in f.readlines():
                    line = line.strip()
                    if re.search(r"umask \d{3}", line):
                        if int(re.search(r"umask \d{3}", line).group().split(" ")[1]) >= 27:
                            num += 1
            if num == 4:
                result["code"] = 0
            else:
                result["code"] = 1
        except Exception as e:
            result["code"] = 1
        self.func_result.append(result)

    def check_baseline(self):
        try:
            sys_version = self.get_sys()
            if sys_version.startswith("7"):
                self.check_max_user_loggin()
                self.password_lock()
                self.shell_TMOUT()
                self.no_passwd()
                self.ssh_timeout()
                self.mem_check()
                self.grub_file_user_sys()
                self.open_ASLR()
                self.session_loggings_sys()
                self.admin_log_sys()
                self.logging_exit_log_sys()
                self.scope_log_sys()
                self.max_log_no_del()
                self.user_group_log_sys()
                self.selinux_wirte_sys()
                self.audit_conf_no_auditctl_sys()
                self.close_cron_mail()
                self.check_umask()
                # print(json.dumps(self.checkdata, encoding='utf-8', ensure_ascii=False))
                # print(json.dumps(self.func_result, encoding='utf-8', ensure_ascii=False))
                return self.func_result
            else:
                self.check_max_user_loggin()
                self.no_passwd()
                self.ssh_timeout()
                self.check_umask()
                self.grub_file_user_sys()
                self.open_ASLR()
                self.session_loggings_sys()
                self.admin_log_sys()
                self.logging_exit_log_sys()
                self.scope_log_sys()
                self.max_log_no_del()
                self.user_group_log_sys()
                self.selinux_wirte_sys()
                self.passwd_file_sys6()
                self.group_file_sys6()
                self.audit_conf_no_auditctl_sys()
                self.close_cron_mail()
                # print(json.dumps(self.checkdata, encoding='utf-8', ensure_ascii=False))
                return self.func_result
        except:
            print("check_allerror")

    def update_option(self,param):
        result = {}
        func_name = param['func_name']
        try:
            agentClass = update_opetion()
            if hasattr(agentClass, func_name) == True:
                mtd = getattr(agentClass, func_name)
                if mtd() == 0:
                    result["msg"] = func_name
                    result["code"] = 0
        except:
            result["msg"] = func_name
            result["code"] = 1
        return result

    def get_sys(self):
        try:
            redhat = os.popen("cat /etc/redhat-release").read()
            if redhat:
                sys_version = re.search("release [0-9].[0-9]", redhat)
                return sys_version.group().split(' ')[1]
        except:
            print("error")

    """
    以上方法为检查方法，以下为更新方法
    """
class update_opetion(object):

    def __init__(self):
        self.result = ''

    def update_max_user_loggin(self):
        num = 0
        try:
            with open("/etc/ssh/sshd_config") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line.startswith("MaxAuthTries"):
                        num += 1
            if num == 1:
                os.system("sed -i 's/^MaxAuthTries.*/MaxAuthTries 4/' /etc/ssh/sshd_config")
            elif num == 0:
                os.system("sed -i '$a MaxAuthTries 4' /etc/ssh/sshd_config")
            return 0
        except:
            return 1


    def update_password_lock(self):
        password_auth_num = 0
        try:
            with open("/etc/pam.d/password-auth") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line == "auth required pam_faillock.so preauth audit silent deny=5 unlock_time=900" or line == "auth [success=1 default=bad] pam_unix.so" or line == "auth [default=die] pam_faillock.so authfail audit deny=5 unlock_time=900" or line == "auth sufficient pam_faillock.so authsucc audit deny=5 unlock_time=900":
                        password_auth_num += 1
            with open("/etc/pam.d/system-auth") as f:
                for line in f.readlines():
                    line = line.strip()
                    if line == "auth required pam_faillock.so preauth audit silent deny=5 unlock_time=900" or line == "auth [success=1 default=bad] pam_unix.so" or line == "auth [default=die] pam_faillock.so authfail audit deny=5 unlock_time=900" or line == "auth sufficient pam_faillock.so authsucc audit deny=5 unlock_time=900":
                        password_auth_num += 1
            if password_auth_num != 0:
                os.system("sed -i '/^auth.*required.*pam_faillock.so preauth audit silent deny=5 unlock_time=900$\|^auth.*[success=1 default=bad].*pam_unix.so$\|^auth.*[default=die].*pam_faillock.so authfail audit deny=5 unlock_time=900$\|^auth.*sufficient.*pam_faillock.so authsucc audit deny=5 unlock_time=900$/d'  /etc/pam.d/password-auth")
                line_result = os.popen("egrep -w -n '^auth' /etc/pam.d/password-auth |awk 'END {print}' |awk -F : '{print $1}'").read().strip('\n')
                os.system("""  sed -i "%sa auth required pam_faillock.so preauth audit silent deny=5 unlock_time=900\\nauth [success=1 default=bad] pam_unix.so\\nauth [default=die] pam_faillock.so authfail audit deny=5 unlock_time=900\\nauth sufficient pam_faillock.so authsucc audit deny=5 unlock_time=900" /etc/pam.d/password-auth """ % line_result)
                os.system("sed -i '/^auth.*required.*pam_faillock.so preauth audit silent deny=5 unlock_time=900$\|^auth.*[success=1 default=bad].*pam_unix.so$\|^auth.*[default=die].*pam_faillock.so authfail audit deny=5 unlock_time=900$\|^auth.*sufficient.*pam_faillock.so authsucc audit deny=5 unlock_time=900$/d'  /etc/pam.d/system-auth")
                line_result2 = os.popen("egrep -w -n '^auth' /etc/pam.d/system-auth |awk 'END {print}' |awk -F : '{print $1}'").read().strip('\n')
                os.system("""  sed -i "%sa auth required pam_faillock.so preauth audit silent deny=5 unlock_time=900\\nauth [success=1 default=bad] pam_unix.so\\nauth [default=die] pam_faillock.so authfail audit deny=5 unlock_time=900\\nauth sufficient pam_faillock.so authsucc audit deny=5 unlock_time=900" /etc/pam.d/system-auth """ % line_result2)
            else:
                line_result = os.popen("egrep -w -n '^auth' /etc/pam.d/password-auth |awk 'END {print}' |awk -F : '{print $1}'").read().strip('\n')
                line_result2 = os.popen("egrep -w -n '^auth' /etc/pam.d/system-auth |awk 'END {print}' |awk -F : '{print $1}'").read().strip('\n')
                os.system("""  sed -i "%sa auth required pam_faillock.so preauth audit silent deny=5 unlock_time=900\\nauth [success=1 default=bad] pam_unix.so\\nauth [default=die] pam_faillock.so authfail audit deny=5 unlock_time=900\\nauth sufficient pam_faillock.so authsucc audit deny=5 unlock_time=900" /etc/pam.d/password-auth """ % line_result)
                os.system("""  sed -i "%sa auth required pam_faillock.so preauth audit silent deny=5 unlock_time=900\\nauth [success=1 default=bad] pam_unix.so\\nauth [default=die] pam_faillock.so authfail audit deny=5 unlock_time=900\\nauth sufficient pam_faillock.so authsucc audit deny=5 unlock_time=900" /etc/pam.d/system-auth """  % line_result2)
            return 0
        except:
            return 1

    def update_shell_TMOUT(self):
        try:
            os.system(""" sed -i /^TMOUT/d /etc/bashrc  /etc/profile && sed -i '$a TMOUT=600' /etc/profile /etc/bashrc """)
            return 0
        except:
            return 1

    def update_no_passwd(self):
        try:
            os.system(""" sed -i /^PermitEmptyPasswords/d /etc/ssh/sshd_config && sed -i '$a PermitEmptyPasswords no' /etc/ssh/sshd_config """)
            return 0
        except:
            return 1

    def update_ssh_timeout(self):
        try:
            # os.system(""" sed -i /^ClientAliveInterval\|^ClientAliveCountMax/d /etc/ssh/sshd_config  && sed -i '$a ClientAliveInterval 300\\nClientAliveCountMax 0' //etc/ssh/sshd_config""")
            os.system(""" sed -i '/^ClientAliveInterval/d; /^ClientAliveCountMax/d' /etc/ssh/sshd_config  && sed -i '$a ClientAliveInterval 300\\nClientAliveCountMax 0' //etc/ssh/sshd_config""")
            return 0
        except:
            return 1

    def update_check_umask(self):
        try:
            result = os.system("""sed  -i 's/umask [0-9]\{1,3\}/umask 027/g' /etc/profile /etc/bashrc && source /etc/profile && source /etc/bashrc""")
            if result == 0:
                return 0
            else:
                return 1
        except:
            return 1

    def update_mem_check(self):
        try:
            os.system("""sed  -i '/^\* hard core.*/d' /etc/security/limits.conf &&  sed -i '$a \* hard core 0' /etc/security/limits.conf""")
            os.system("""sed  -i '/^fs\.suid_dumpable/d' /etc/sysctl.conf &&  sed -i '$a fs.suid_dumpable = 0' /etc/sysctl.conf""")
            return 0
        except:
            return 1

    def update_grub_file_user_sys(self):
        sys_version = check().get_sys()
        try:
            if sys_version.startswith("7"):
                if not os.path.exists("/boot/grub2/grub.cfg"):
                    os.system("grub2-mkconfig -o /boot/grub2/grub.cfg")
                os.system("chown root:root /boot/grub2/grub.cfg && chmod og-rwx /boot/grub2/grub.cfg")
            else:
                os.system("chown root:root /boot/grub/grub.conf && chmod og-rwx /boot/grub/grub.conf")
            return 0
        except:
            return 1

    def update_open_ASLR(self):
        try:
            os.system(""" sed -i /^kernel.randomize_va_space/d /etc/sysctl.conf && sed -i '$a kernel.randomize_va_space = 2' /etc/sysctl.conf""")
            return 0
        except:
            return 1

    def update_session_loggings_sys(self):
        sys_version = check().get_sys()
        try:
            if sys_version.startswith("7"):
                if not os.path.exists("/etc/audit/rules.d/audit.rules") or os.path.getsize("/etc/audit/rules.d/audit.rules") == 0:
                    with open("/etc/audit/rules.d/audit.rules", "w") as f:
                        f.write("-w /var/run/utmp -p wa -k session\n-w /var/log/wtmp -p wa -k logins\n-w /var/log/btmp -p wa -k logins\n")
                else:
                    os.system("sed -i '/^-w \/var\/run\/utmp -p wa -k\|^-w \/var\/log\/wtmp -p wa -k\|^-w \/var\/log\/btmp -p wa -k/d'  /etc/audit/rules.d/audit.rules && sed -i '$i -w /var/run/utmp -p wa -k session\\n-w /var/log/wtmp -p wa -k logins\\n-w /var/log/btmp -p wa -k logins' /etc/audit/rules.d/audit.rules")
            else:
                if not os.path.exists("/etc/audit/audit.rules") or os.path.getsize("/etc/audit/audit.rules") == 0:
                    with open("/etc/audit/audit.rules", "w") as f:
                        f.write("-w /var/run/utmp -p wa -k session\n-w /var/log/wtmp -p wa -k session\n-w /var/log/btmp -p wa -k session\n")
                else:
                    os.system("sed -i '/^-w \/var\/run\/utmp -p wa -k\|^-w \/var\/log\/wtmp -p wa -k\|^-w \/var\/log\/btmp -p wa -k/d'  /etc/audit/audit.rules && sed -i '$i -w /var/run/utmp -p wa -k session\\n-w /var/log/wtmp -p wa -k session\\n-w /var/log/btmp -p wa -k session' /etc/audit/audit.rules")
            return 0
        except:
            return 1

    def update_admin_log_sys(self):
        sys_version = check().get_sys()
        try:
            if sys_version.startswith("7"):
                os.system(" sed -i '/^-w \/var\/log\/sudo.log -p wa -k actions/d' /etc/audit/rules.d/audit.rules && sed -i '$i -w /var/log/sudo.log -p wa -k actions'  /etc/audit/rules.d/audit.rules")
            else:
                os.system(" sed -i '/^-w \/var\/log\/sudo.log -p wa -k actions/d' /etc/audit/audit.rules && sed -i '$i -w /var/log/sudo.log -p wa -k actions'  /etc/audit/audit.rules")
            return 0
        except:
            return 1

    def update_logging_exit_log_sys(self):
        sys_version = check().get_sys()
        try:
            if sys_version.startswith("7"):
                os.system("sed -i '/^-w \/var\/log\/lastlog -p wa -k\|^-w \/var\/run\/faillock\/ -p wa -k logins/d' /etc/audit/rules.d/audit.rules && sed -i '$i -w /var/log/lastlog -p wa -k logins\\n-w /var/run/faillock/ -p wa -k logins'  /etc/audit/rules.d/audit.rules")
            else:
                os.system(" sed -i '/^-w \/var\/log\/lastlog -p wa -k\|^-w \/var\/run\/faillock\/ -p wa -k logins/d' /etc/audit/audit.rules && sed -i '$i -w /var/log/lastlog -p wa -k logins\\n-w /var/run/faillock/ -p wa -k logins'  /etc/audit/audit.rules")
            return 0
        except:
            return 1

    def update_scope_log_sys(self):
        sys_version = check().get_sys()
        try:
            if sys_version.startswith("7"):
                os.system(" sed -i '/^-w \/etc\/sudoers -p wa -k scope\|^-w \/etc\/sudoers.d\/ -p wa -k scope/d' /etc/audit/rules.d/audit.rules && sed -i '$i -w /etc/sudoers -p wa -k scope\\n-w /etc/sudoers.d/ -p wa -k scope'  /etc/audit/rules.d/audit.rules")
            else:
                os.system(" sed -i '/^-w \/etc\/sudoers -p wa -k scope\|^-w \/etc\/sudoers.d -p wa -k scope/d' /etc/audit/audit.rules && sed -i '$i -w /etc/sudoers -p wa -k scope\\n-w /etc/sudoers.d -p wa -k scope'  /etc/audit/audit.rules")
            return 0
        except:
            return 1

    def update_max_log_no_del(self):
        try:
            os.system(""" sed -i /^max_log_file_action/d /etc/audit/auditd.conf && sed -i '$i max_log_file_action = keep_logs' /etc/audit/auditd.conf""")
            return 0
        except:
            return 1

    def update_user_group_log_sys(self):
        sys_version = check().get_sys()
        try:
            if sys_version.startswith("7"):
                os.system("sed -i '/^-w \/etc\/group -p wa -k\|^-w \/etc\/passwd -p wa -k\|^-w \/etc\/gshadow -p wa -k\|^-w \/etc\/shadow -p wa -k\|^-w \/etc\/security\/opasswd -p wa -k/d' /etc/audit/rules.d/audit.rules && sed -i '$i -w /etc/group -p wa -k identity\\n-w /etc/passwd -p wa -k identity\\n-w /etc/gshadow -p wa -k identity\\n-w /etc/shadow -p wa -k identity\\n-w /etc/security/opasswd -p wa -k identity' /etc/audit/rules.d/audit.rules")
            else:
                os.system("sed -i '/^-w \/etc\/group -p wa -k\|^-w \/etc\/passwd -p wa -k\|^-w \/etc\/gshadow -p wa -k\|^-w \/etc\/shadow -p wa -k\|^-w \/etc\/security\/opasswd -p wa -k/d' /etc/audit/audit.rules && sed -i '$i -w /etc/group -p wa -k identity\\n-w /etc/passwd -p wa -k identity\\n-w /etc/gshadow -p wa -k identity\\n-w /etc/shadow -p wa -k identity\\n-w /etc/security/opasswd -p wa -k identity' /etc/audit/audit.rules")
            return 0
        except:
            return 1

    def update_selinux_wirte_sys(self):
        sys_version = check().get_sys()
        try:
            if sys_version.startswith("7"):
                os.system("sed -i '/^-w \/etc\/selinux\/ -p wa -k MAC/d' /etc/audit/rules.d/audit.rules && sed -i '$i -w /etc/selinux/ -p wa -k MAC-policy\\n-w /usr/share/selinux/ -p wa -k MAC-policy' /etc/audit/rules.d/audit.rules")
                return "yesyes!"
            else:
                os.system("sed -i '/^-w \/etc\/selinux\/ -p wa -k MAC\|^-w \/usr\/share\/selinux\/ -p wa -k MAC/d' /etc/audit/audit.rules && sed -i '$i -w /etc/selinux/ -p wa -k MAC-policy' /etc/audit/audit.rules")
            return 0
        except:
            return 1

    def update_audit_conf_no_auditctl_sys(self):
        sys_version = check().get_sys()
        try:
            if sys_version.startswith("7"):
                os.system("sed -i '/^-e/d' /etc/audit/rules.d/audit.rules &&  sed -i '$a -e 2' /etc/audit/rules.d/audit.rules")
            else:
                os.system("sed -i '/^-e/d' /etc/audit/audit.rules &&  sed -i '$a -e 2' /etc/audit/audit.rules")
            return 0
        except:
            return 1

    def update_passwd_file_sys6(self):
        try:
            os.system("chown root:root /etc/passwd- && chmod 600 /etc/passwd-")
            return 0
        except:
            return 1

    def update_group_file_sys6(self):
        try:
            os.system("chown root:root /etc/group- && chmod 600 /etc/group-")
            return 0
        except:
            return 1

    def update_close_cron_mail(self):
        sys_version = check().get_sys()
        try:
            if os.path.exists("/etc/crontab"):
                os.system("""sed -i '/^MAILTO=/d' /etc/crontab && sed -i '/SHELL/aMAILTO=""' /etc/crontab""")
            else:
                with open("/etc/crontab", "w") as f:
                    f.write("SHELL=/bin/bash\nMAILTO=""\nPATH=/sbin:/bin:/usr/sbin:/usr/bin\nHOME=/")
                os.system("""chmod 644 /etc/crontab""")
            if sys_version.startswith("7"):
                result = os.system("systemctl restart crond")
            else:
                result = os.system("/etc/init.d/crond restart")
            return result
        except:
            return 1

if __name__ == '__main__':
   pass
